#ovaj fajl prevesti naredbom gcc saberi_main.s
#NAPOMENA: Pri koriscanju data sekcije neki kompajleri podrazumevano 
#generisu takozvane deljene PIE objekte koji zahtevaju drugaciji
#pristup alociranoj memoriji u data sekciji. Da bi se onemogucilo
#kreiranje PIE objekta potrebno je prilikom kompilacije dodati 
#opciju -no-pie (gcc saberi_main.s -no-pie).

#koristimo intelovu neprefiksnu sintaksu
.intel_syntax noprefix

#pocinjemo sekciju podataka
.data
        #Alociramo i inicijalizujemo memoriju na segmentu podataka za
	#smestanje format niski za scanf i printf. Potrebno je labelom
	#oznaciti memoriju na kojoj su smestene niske kako bismo kasnije
	#mogli da im pristupimo i prosledimo ih kao argumente funkcija 
	#scanf i printf.
	s1: .asciz "%d %d"  	
	s2: .asciz "Zbir: %d\n"

#pocinjemo sekciju koda
.text

#neophodno je da labela main koja predstavlja naziv funkcije main bude 
#globalno vidljiva kako bi linker mogao da razresi poziv odgovarajuce 
#funkcije iz drugog modula programa (crt0.o je modul koji obezbedjuje 
#kompajler i koji se podrazumevano linkuje sa svakim C programom; u 
#njemu se nalazi funkcija _start od koje zapravo pocinje izvrsavanje 
#programa, i koja poziva funkciju main)
.global main

saberi:
	enter 0,0
	mov eax, edi
	add eax, esi
	leave
	ret

main:
	#prolog funkcije
	#rezervisemo 8 bajtova za smestanje lokalnih promenljivih 
	#funkcije main (za dve 4-bajtne promenljive tipa int)
	enter 8, 0

	#pripremamo argumente za poziv funkcije scanf
	#ucitavamo adresu koju oznacava labela s1 u registar rdi (1. argument)
	lea rdi, s1
	#ucitavamo adresu prve lokalne promenljive u registar rsi (2. argument)
	lea rsi, [rbp-4]
	#ucitavamo adresu druge lokalne promenljive u registar rdx (3. argument)
	lea rdx, [rbp-8]

	#pri pozivu funkcija sa promenljivim brojem argumenata (kao sto su scanf
	#i printf) u rax registru se prenosi broj argumenata koji se prosledjuje
	#u vektorskim registrima - kod nas je to uvek 0
	mov rax, 0

	#--------------------------------------------------------------------------
	#pri call instrukciji, na stek se stavlja ret adresa (8 bajtova)
	#pri enter n, 0 instrukciji, na stek se stavlja adresa pocetka okvira steka
	#pozivajuce funkcije (8 bajtova) i jos n bajtova za lokalne promenljive
	#-> ukoliko je vrh steka bio deljiv sa 16 pre call instrukcije za main, 
	#   nakon nje (tj. posle enter n, 0 instrukcije) imamo jos 16 + n bajtova 
	#   na steku
	#-> trenutna adresa u rsp registru je 16k + (16 + n), pa je neophodno da 
	#   ukoliko n nije deljivo sa 16 pomerimo vrh steka za koliko bajtova je 
	#   neophodno da bi adresa sadrzana u rsp bila deljiva sa 16
	#--------------------------------------------------------------------------
	#da bi adresa vrha steka pri pozivu funkcije scanf bila deljiva sa 16
	#moramo da pomerimo pokazivac vrha steka za jos 8 bajtova na gore
	sub rsp, 8

	#pozivamo funkciju scanf (njoj odgovara istoimena labela iz C standardne
	#biblioteke (C standardna biblioteka se podrazumevano linkuje sa svakim 
	#C programom)
	call scanf

	#pripremamo argumente za poziv funkcije saberi
	#ucitavamo vrednost koja je upisana na adresu prve lokalne promenljive u registar edi (1.argument)
	mov edi, [rbp-4]
	#ucitavamo vrednost koja je upisana na dresu druge lokalne promenljive u registar esi (2.argument)
	mov esi, [rbp-8]

	call saberi

	#pripremamo argumente za poziv funkcije printf
	#ucitavamo adresu koju oznacava labela s2 u registar rdi (1. argument)
	lea rdi, s2
	#ucitavamo vrednost povratnu vrednost funkcije saberi u registar esi (2.argument)
	mov esi, eax

	#pri pozivu funkcija sa promenljivim brojem argumenata (kao sto su scanf
	#i printf) u rax registru se prenosi broj argumenata koji se prosledjuje
	#u vektorskim registrima - kod nas je to uvek 0
	mov rax, 0

	#pozivamo funkciju printf (njoj odgovara istoimena labela iz C standardne
	#biblioteke (C standardna biblioteka se podrazumevano linkuje sa svakim 
	#C programom)
	call printf

	#kao povratnu vrednost main-a vracamo 0
	mov eax, 0

	#epilog funkcije
	#NAPOMENA: deo steka koji koristimo u funkciji (dodatnih 16 bajtova) ce se 
	#ispravno 'isprazniti' (ponistiti) instrukcijom leave koja izjednacava vrh
	#steka sa pocetkom stek okvira (leave <=> mov rsp, rbp ; pop rbp)
	leave

	ret

